# Google::Search
# $Id: google.rb,v 1.10 2002/04/15 18:25:52 ianmacd Exp $
# 
# Version : 0.2.0
# Author  : Ian Macdonald <ian@caliban.org>
#
# Copyright (c) 2002 Ian Macdonald
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

=begin

= NAME
  Google::Search
= SYNOPSIS
  require 'google'

  KEY = File.open("#{ENV['HOME']}/.google_key") {|kf| kf.readline.chomp}

  query = ARGV.shift || 'ruby programming language'
  google = Google::Search.new(KEY)

  i = 0
  q = google.search(query)
  q.response['resultElements'].each do |result|
    printf "\nResult # %d\n\n", i += 1
    result.each do |key|
      printf("%s = %s\n", key, result.attribute[key])
    end
  end

  puts '---------------------------------'
  printf "Estimated number of results is %d.\n",
          q.response['estimatedTotalResultsCount']
  printf "Your query took %6f seconds.\n", q.response['searchTime']

= DESCRIPTION
Google::Search allows you to programmatically query the Google search-engine
((<at www.google.com|URL:http://www.google.com>)). It is currently in the
alpha stage and the interface is liable to change at any time.
= CLASS METHODS
--- Search.new(key)
    This constructs a new Google search object. The key parameter is the
    key that Google assigned to you when you registered for the Web API
    download. If you don't yet have a key, go to
    ((<Google|URL:https://www.google.com/accounts/NewAccount?continue=http://api.google.com/createkey&followup=http://api.google.com/createkey>))
    and obtain one.
= INSTANCE METHODS
--- Search#search(query, start, max, filter, restrict, safe, lr, ie, oe)
    This performs a standard Google query search. Only the ((|query|))
    parameter is mandatory.

    The meaning of the other parameters can be obtained from section 2.1 of
    the APIs_Reference.html file that came with your Google API archive.

    This method returns a Google::Response object, the members of which are
    described in section 3.1 of APIs_Reference.html.

    The ((|resultElements|)) element is an array of Google::ResultElement
    objects. Elements of the Google::ResultElement object are described in
    section 3.2 of APIs_Reference.html.
--- Search#spell(phrase)
    This attempts to perform a Google spell-check on ((|phrase|)). If Google
    has no spelling suggestions to make, ((*nil*)) is returned. Otherwise,
    a String is returned.
--- Search#cache(url)
    This attempts to retrieve the cached Google copy of the page corresponding
    to ((|url|)). If Google has not cached the URL in question, a page
    containing a message to this effect will be returned by the search-engine.

    This method always returns a String.
= ENVIRONMENT
* HTTP_PROXY or http_proxy
  If this is defined, the named system will be used
  as an HTTP proxy.
= AUTHOR
  Written by Ian Macdonald <ian@caliban.org>.
= COPYRIGHT
  Copyright (c) 2002 Ian Macdonald
  This is free software; see the source for copying conditions.
  There is NO warranty; not even for MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.
= SEE ALSO
  http://www.caliban.org/ruby
  http://www.google.com/
= BUGS
  Send all bug reports, enhancement requests and patches to the
  author.
= HISTORY
  $Id: google.rb,v 1.10 2002/04/15 18:25:52 ianmacd Exp $
=end

require 'soap/driver'

module Google

VERSION  = '0.2.0'
ENDPOINT = 'http://api.google.com/search/beta2'
NS = 'urn:GoogleSearch'
PROXY = ENV['HTTP_PROXY'] || ENV['http_proxy'] || nil
Response = Struct.new('Response', :directoryCategories, :documentFiltering,
  :endIndex, :estimatedTotalResultsCount, :estimateIsExact, :searchComments,
  :searchQuery, :searchTime, :searchTips, :startIndex, :resultElements)


class Search
  attr_reader :response
  @@soap = SOAP::Driver.new(nil, nil, NS, ENDPOINT, PROXY)
  @@search_method = @@spell_method = @@cache_method = false

  def initialize(k)
    @key = k
  end

  def search(query, start=0, max=10, filter=false, restrict=nil, safe=false,
	     lr=nil, ie='latin1', oe='latin1')
    search_method = 'doGoogleSearch'
    if ! @@search_method
      @@soap.addMethodWithSOAPAction(
	search_method, NS, 'key', 'q', 'start', 'maxResults', 'filter',
	'restrict', 'safeSearch', 'lr', 'ie', 'oe')
      @@search_method = true
    end
    r = @@soap.send(search_method, 
      @key, query, start, max, filter, restrict, safe, lr, ie, oe)
    @response = Google::Response.new
    fields = %w(directoryCategories documentFiltering endIndex
      estimatedTotalResultsCount directoryCategories documentFiltering
      endIndex estimatedTotalResultsCount estimateIsExact searchComments
      searchQuery searchTime searchTips startIndex)
    fields.each {|field| @response[field] = r.send(field)}
    @response.resultElements = r.resultElements.collect {|e|
      Google::ResultElement.new(e)
    }
    self
  end

  def spell(phrase)
    spell_method = 'doSpellingSuggestion'
    if ! @@spell_method
      @@soap.addMethodWithSOAPAction(spell_method, NS, 'key', 'phrase')
      @@spell_method = true
    end
    r = @@soap.send(spell_method, @key, phrase)
    r if r.type == String
  end

  def cache(url)
    cache_method = 'doGetCachedPage'
    if ! @@cache_method
      @@soap.addMethodWithSOAPAction(cache_method, NS, 'key', 'phrase')
      @@cache_method = true
    end
    # decode Base64-encoded response
    @@soap.send(cache_method, @key, url).unpack("m")
  end
end

ResultElement = Struct.new('ResultElement', :url, :snippet, :title,
  :cachedSize, :relatedInformationPresent, :directoryTitle, :summary,
  :hostName, :directoryName, :directoryEncoding)
class ResultElement
  def attribute
    self
  end

  def each(&block)
    self.members.each(&block)
  end

  def initialize(element)
    self.url = element.URL
    self.snippet = element.snippet
    self.title = element.title
    self.cachedSize = element.cachedSize
    self.relatedInformationPresent = element.relatedInformationPresent

    if element.directoryTitle.type == String
      self.directoryTitle = element.directoryTitle
    end
    if element.summary.type == String
      self.summary = element.summary
    end
    if element.hostName.type == String
      self.hostName = element.hostName
    end
    if element.directoryCategory.fullViewableName.type == String
      self.directoryName = element.directoryCategory.fullViewableName
    end
    if element.directoryCategory.specialEncoding == String
      self.directoryEncoding = element.directoryCategory.specialEncoding
    end
  end
end

end
